(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
        typeof define === 'function' && define.amd ? define(['exports'], factory) :
        (factory((global.PARichEditor = {})));
}(this, (function (exports) {
    'use strict';

    var _extends = Object.assign || function (target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }
        return target;
    };

    var defaultParagraphSeparatorString = 'defaultParagraphSeparator';
    var formatBlock = 'formatBlock';
    var oldForeColor = 'rgb(0, 0, 0)';
    var oldRange;
    var actions;
    var settings;
    var contentElm;
    var addEventListener = function addEventListener(parent, type, listener) {
        return parent.addEventListener(type, listener);
    };
    var appendChild = function appendChild(parent, child) {
        return parent.appendChild(child);
    };
    var createElement = function createElement(tag) {
        return document.createElement(tag);
    };
    var queryCommandState = function queryCommandState(command) {
        return document.queryCommandState(command);
    };
    var queryCommandValue = function queryCommandValue(command) {
        return document.queryCommandValue(command);
    };

    var exec = function exec(command) {
        var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
        return document.execCommand(command, false, value);
    };

    var getSelectionContainerElem = function getSelectionContainerElem(range) {
        range = range || window.getSelection()
            .getRangeAt(0);
        var elem = void 0;
        if (range) {
            elem = range.commonAncestorContainer;
            return elem.nodeType === 1 ? elem : elem.parentNode;
        }
    };

    var defaultActions = {
        bold: {
            name: 'BOLD',
            icon: '<b>B</b>',
            title: 'Bold',
            state: function state() {
                    var isSelected = queryCommandState('bold');
                    return {
                        selected: isSelected
                    };
                },
                result: function result() {
                    return exec('bold');
                }
        },
        italic: {
            name: 'ITALIC',
            icon: '<i>I</i>',
            title: 'Italic',
            state: function state() {

                    var isSelected = queryCommandState('italic');
                    return {
                        selected: isSelected
                    };
                },
                result: function result() {
                    return exec('italic');
                }
        },
        underline: {
            name: 'UNDERLINE',
            icon: '<u>U</u>',
            title: 'Underline',
            state: function state() {
                    var isSelected = queryCommandState('underline');
                    return {
                        selected: isSelected
                    };
                },
                result: function result() {
                    return exec('underline');
                }
        },
        foreColor: {
            name: 'FOREGROUND',
            icon: '<b><font color="red">A</font></b>',
            title: 'foreColor',
            state: function state() {
                    var isSelected = queryCommandValue('foreColor') == 'rgb(255, 0, 0)';
                    return {
                        selected: isSelected
                    };
                },
                result: function result() {
                    var res = false;
                    var currColor = queryCommandValue('foreColor');
                    exec("styleWithCSS", true);
                    if ('rgb(255, 0, 0)' != currColor) {
                        //oldForeColor = currColor;不记录设置之前的字体颜色，取消标红统一改为黑色字体
                        res = exec('foreColor', 'rgb(255, 0, 0)');
                    } else {
                        res = exec('foreColor', oldForeColor);
                    }
                    exec("styleWithCSS", false);
                    return res;
                }
        },
        backColor: {
            name: 'BACKGROUND',
            icon: '<b>A<font color="yellow">&#9635</font></b>',
            title: 'backColor',
            state: function state() {
                    var isSelected = false;
                    var currColor = queryCommandValue('backColor');
                    if ('rgb(255, 255, 0)' == currColor) {
                        isSelected = true;
                    } else if ('rgba(0, 0, 0, 0)' == currColor) { //跨段落选择可能获取到的是rgba(0, 0, 0, 0)，遍历查看是否标黄
                        var node = window.getSelection().anchorNode;
                        while (node && (!node.style || !node.style.backgroundColor)) {
                            node = node.parentNode;
                        }
                        if (node && 'rgb(255, 255, 0)' == node.style.backgroundColor) {
                            isSelected = true;
                        }
                    }
                    return {
                        selected: isSelected
                    };
                },
                result: function result() {
                    var res = false;
                    var stateResult = defaultActions.backColor.state();
                    exec("styleWithCSS", true);
                    if (stateResult.selected) {
                        res = exec('backColor', 'rgba(0, 0, 0, 0.01)'); //设置rgba(0, 0, 0, 0)不生效
                    } else {
                        res = exec('backColor', 'rgb(255, 255, 0)');
                    }
                    exec("styleWithCSS", false);
                    return res;
                }
        },
        olist: {
            name: 'ORDEREDLIST',
            icon: '&#35;',
            title: 'Ordered List',
            state: function state() {
                    var isSelected = queryCommandState('insertOrderedList');
                    return {
                        selected: isSelected
                    };
                },
                result: function result() {
                    var res = exec('insertOrderedList');
                    var stateResult = defaultActions.olist.state();
                    if (stateResult.selected) {
                       setFocusNodeRangeEnd();
                    }
                    return res;
                }
        },
        ulist: {
            name: 'NORMALLIST',
            icon: '&#8226;',
            title: 'Unordered List',
            state: function state() {
                    var isSelected = queryCommandState('insertUnorderedList');
                    return {
                        selected: isSelected
                    };
                },
                result: function result() {
                    var res = exec('insertUnorderedList');
                    var stateResult = defaultActions.ulist.state();
                    if (stateResult.selected) {
                        setFocusNodeRangeEnd();
                    }
                    return res;
                }
        },
        link: {
            name: 'LINK',
            icon: '&#128279;',
            title: 'Link',
            state: function state() {
                    var elem = getSelectionContainerElem();
                    var result = {};
                    result.selected = false;
                    if (elem) {
                        result.selected = elem.nodeName === 'A';
                        if (result.selected) {
                            result.url = elem.getAttribute('href');
                            result.text = elem.text;
                        }
                    }

                    return result;
                },
                result: function result() {
                    var stateResult = defaultActions.link.state();
                    var selectText = window.getSelection().toString();
                    if (selectText) {
                        if (selectText != stateResult.text) {
                            stateResult.url = '';
                        }
                        stateResult.text = selectText;
                    } else {
                        stateResult.url = '';
                        stateResult.text = '';
                    }
                    saveRange();
                    settings.onClickResult(defaultActions.link.name, JSON.stringify(stateResult));
                }
        },
        image: {
            name: 'image',
            icon: '&#128247;',
            title: 'Image',
            result: function result() {
                var url = window.prompt('Enter the image URL');
                if (url) exec('insertImage', url);
            }
        }
    };

    var setFocusNodeRangeEnd = function setFocusNodeRangeEnd(node) {
        var sel = document.getSelection();
        if(!node){
           node = sel.focusNode;
        }
        if (node) {
            var range = document.createRange();
            range.selectNodeContents(node);
            range.collapse(false);
            sel.removeAllRanges();
            sel.addRange(range);
        }
    };

    var defaultClasses = {
        actionbar: 'PARichEditor-actionbar',
        button: 'PARichEditor-button',
        content: 'PARichEditor-content',
        selected: 'PARichEditor-button-selected'
    };

    var init = function init(initSettings) {
        settings = initSettings;
        actions = settings.actions ? settings.actions.map(function (action) {
                if (typeof action === 'string') return defaultActions[action];
                else if (defaultActions[action.name]) return _extends({}, defaultActions[action.name], action);
                return action;
            }) : Object.keys(defaultActions)
            .map(function (action) {
                return defaultActions[action];
            });

        var classes = _extends({}, defaultClasses, settings.classes);

        var defaultParagraphSeparator = settings[defaultParagraphSeparatorString] || 'div';

        var actionbar = createElement('div');
        actionbar.className = classes.actionbar;
        appendChild(settings.element, actionbar);

        contentElm = settings.element.content = createElement('div');
        contentElm.id = "id_editor_content"
        contentElm.contentEditable = true;
        contentElm.className = classes.content;
        contentElm.oninput = function (_ref) {
            var firstChild = _ref.target.firstChild;

            if (firstChild && firstChild.nodeType === 3) {
                exec(formatBlock, '<' + defaultParagraphSeparator + '>');
            } else if (contentElm.innerHTML === '<br>') {
                contentElm.innerHTML = '';
            }
            settings.onHtmlChange(contentElm.innerHTML);
        };

        appendChild(settings.element, contentElm);

        actions.forEach(function (action) {
            var button = createElement('button');
            button.className = classes.button;
            button.innerHTML = action.icon;
            button.title = action.title;
            button.id = "id_" + action.name
            button.setAttribute('type', 'text');
            button.onclick = function () {
                return action.result() && contentElm.focus();
            };

            if (action.state) {
                var handler = function handler() {
                    var stateResult = action.state();
                    var result = button.classList[stateResult.selected ? 'add' : 'remove'](classes.selected);
                    settings.onStateChange(action.name, JSON.stringify(stateResult));
                    return result;
                };
                addEventListener(contentElm, 'keyup', handler);
                addEventListener(contentElm, 'mouseup', handler);
                document.addEventListener("selectionchange", handler);
                addEventListener(button, 'click', handler);
            }

            appendChild(actionbar, button);
        });

        if (settings.styleWithCSS) exec('styleWithCSS');
        exec(defaultParagraphSeparatorString, defaultParagraphSeparator);

        return settings.element;
    };

    var doClickActionButton = function doClickActionButton(actionName) {
        var button = document.getElementById('id_' + actionName);
        if (button) {
            return button.click();
        }
        return false;
    };

    var setPlaceholder = function setPlaceholder(placeholder) {
        contentElm.setAttribute("placeholder", placeholder);
    };
    var setHtml = function setHtml(html) {
        contentElm.innerHTML = decodeURIComponent(html.replace(/\+/g, '%20'));
    };
    var getHtml = function getHtml() {
        return contentElm.innerHTML;
    };
    var insertHTML = function (html) {
        html = decodeURIComponent(html.replace(/\+/g, '%20'));
        document.execCommand('insertHTML', false, html);
    };
    var setHeight = function setHeight(height) {
        contentElm.style.height = height;
    };

    var insertLink = function insertLink(url, text) {
        reSetOldRange();
        exec('insertHTML', '<a href="' + url + '">' + text + '</a>');
    };

    var saveRange = function saveRange() {
        oldRange = document.getSelection().getRangeAt(0).cloneRange();
    }

    var focus = function focus(){
         contentElm.focus();
    }

    var reSetOldRange = function reSetOldRange() {
        document.getSelection().removeAllRanges();
        document.getSelection().addRange(oldRange);
    };
    var getElement = function getElement() {
        return contentElm;
    };
    var PARichEditor = {
        exec: exec,
        init: init,
        doClickActionButton: doClickActionButton,
        setPlaceholder: setPlaceholder,
        setHtml: setHtml,
        getHtml: getHtml,
        insertHTML: insertHTML,
        setHeight: setHeight,
        insertLink: insertLink,
        reSetOldRange: reSetOldRange,
        getElement: getElement,
        focus:focus,
        setFocusNodeRangeEnd:setFocusNodeRangeEnd
    };

    exports.exec = exec;
    exports.init = init;
    exports.focus = focus;
    exports.doClickActionButton = doClickActionButton;
    exports.setPlaceholder = setPlaceholder;
    exports.setHtml = setHtml;
    exports.getHtml = getHtml;
    exports.insertHTML = insertHTML;
    exports.insertLink = insertLink;
    exports.reSetOldRange = reSetOldRange;
    exports.setHeight = setHeight;
    exports.getElement = getElement;
    exports.setFocusNodeRangeEnd = setFocusNodeRangeEnd;
    exports['default'] = PARichEditor;

    Object.defineProperty(exports, '__esModule', {
        value: true
    });

})));