"use strict";

/**
 * 邮件收发件人折叠
 *  1. 先遍历dom树， 找到所有邮件头节点(根据‘收件人’查找)
 *  2. 对每一个文件头节点，生成影子节点显示（生成ID， class），隐藏真实节点
 */
var fontSizeMap = {};
// 新增部分class名称
var foldClassName = "_mail_fold_";
function hasClass(element, className) {
  className = className
    .replace(/(^[ ]+)|([ ]+$)/g, "")
    .replace(/[ ]{2,}/g, " ")
    .split(" "); // eslint-disable-next-line

  for (var i = 0, ci, cls = element.className; (ci = className[i++]); ) {
    if (!new RegExp("\\b" + ci + "\\b", "i").test(cls)) {
      return false;
    }
  }

  return i - 1 === className.length;
}

function findSender() {
  var SENDER_TAG_REG = /(发件人|From)/i;
  var SENDER_REG = /^(发件人|From)[\s]*[:：]*$/i;
  var ret = []; // 按层次遍历找‘发件人'节点

  var queue = [document.body];

  while (queue.length) {
    var curentNode = queue.shift();

    if (!curentNode) {
      continue;
    }

    var curText = curentNode.innerText.trim();

    if (SENDER_REG.test(curText)) {
      // 如果仅仅包含收件人， 说明当前节点就是目标节点
      ret.push(curentNode);
      continue;
    }

    if (!SENDER_TAG_REG.test(curText)) {
      // 没有找到‘发件人’就不需要处理了
      continue;
    }

    for (var i = 0; i < curentNode.children.length; i++) {
      queue.push(curentNode.children[i]);
    }
  }

  return ret;
}

function isParent(fnode, cnode) {
  var curentNode = cnode.parentNode;

  while (curentNode) {
    if (curentNode === fnode) {
      return true;
    }

    curentNode = curentNode.parentNode;
  }

  return false;
}

function delParent(array) {
  var ret = [];

  for (var i = 0; i < array.length; i++) {
    var item = array[i];
    var findChild = false;

    for (var j = 0; j < array.length; j++) {
      if (i === j) continue;

      if (isParent(item, array[j])) {
        findChild = true;
        break;
      }
    }

    if (!findChild) {
      ret.push(item);
    }
  }

  return ret;
}

function findMailHead() {
  var ret = [];
  var senderList = findSender();
  var checkZhReg = /发件人[\s\S]*(发送时间|日期)[\s\S]*收件人[\s\S]*主题/;
  var checkEnReg = /From[\s\S]*(Sent|Date)[\s\S]*To[\s\S]*Subject/;
  senderList.forEach(function(item) {
    var curentNode = item.parentNode;

    while (curentNode) {
      if (
        curentNode.tagName === "DIV" ||
        hasClass(curentNode, "MsoNormal") ||
        hasClass(curentNode, "MsoPlainText")
      ) {
        var text = curentNode.innerText; // 验证是否包含邮件必要信息

        if (checkZhReg.test(text) || checkEnReg.test(text)) {
          // 这里字体有pt, flex布局有问题, 记录下字体大小
          ret.push(curentNode);

          if (fontSizeMap[getComputedStyle(item).fontSize]) {
            fontSizeMap[getComputedStyle(item).fontSize]++;
          } else {
            fontSizeMap[getComputedStyle(item).fontSize] = 1;
          }
        } // 这里认为’DIV‘ 和 ’。MsoNormal‘ 是终结符， 是否存在其他情况 ？？

        break;
      }

      curentNode = curentNode.parentNode;
    }
  }); // 判断是否存在父子关系的节点， 父节点要去掉

  return delParent(ret);
}

function findFixFont() {
  // flex下投票选出最合适的字体px
  var fixFont;
  var cur = -1;

  for (var key in fontSizeMap) {
    if (fontSizeMap[key] > cur) {
      cur = fontSizeMap[key];
      fixFont = key;
    }
  }
  return fixFont;
}

function boldKey(text) {
  // 加粗冒号前面的文字
  var index = -1;
  index = "".concat(text).indexOf(":");

  if (index === -1) {
    // 考虑到有中文
    index = "".concat(text).indexOf("：");
  }

  if (index === -1) {
    // 考虑到有中文
    return text;
  }

  return "<b>"
    .concat(text.slice(0, index), "</b>")
    .concat(text.slice(index, index + 1), "&nbsp;")
    .concat(text.slice(index + 1));
}

function createShadow(headerList) {
  var createdMap = {};
  headerList.forEach(function(header, index) {
    var id = header.getAttribute("fid"); // id有重复

    if (!id) {
      id = ""
        .concat(index, "_")
        .concat(+new Date(), "_")
        .concat(parseInt(1000 * Math.random(), 10));
      header.setAttribute("fid", id);
    }

    if (createdMap[id]) {
      return;
    }

    createdMap[id] = true; // 获取邮件信息, 简单认为'\n'分割, 后续可能要优化

    if (header.getElementsByTagName("img").length) {
      // 特殊情况， 如果有图片，就不处理了
      return;
    }

    var emailInfo = {};
    var emialTextArray = header.innerText.split("\n");
    if (emialTextArray.length < 3) {
      // 考虑发件人， 收件人， 时间时必须的
      return;
    }
    emialTextArray.forEach(function(text) {
      if (/^(发件人|From)/i.test(text.trim())) {
        emailInfo.from = boldKey(text);
      }

      if (/^(发送时间|日期|Sent|Date)/i.test(text.trim())) {
        emailInfo.date = boldKey(text);
      }
    }); // header.style.display = 'none';
    // 创建影子节点

    var headerHtml = emialTextArray
      .map(function(text) {
        if (text.trim()) {
          return "<div>".concat(boldKey(text), "</div>");
        }

        return "";
      })
      .join("");
    var isEn = navigator.userAgent.indexOf("Mail_En") > -1;
    var moreText = isEn ? "Details" : "详情";
    var lessText = isEn ? "Hide" : "隐藏";
    header.setAttribute("style", "");
    header.innerHTML = "";
    var shadowFull = document.createElement("div");
    var shadowThumb = document.createElement("div");
    shadowThumb.className = foldClassName;
    var fontSize = findFixFont();
    var warpSryle =
      "padding: 10px;line-height: 1.2; background: #EFEFEF; -webkit-tap-highlight-color:transparent;";
    shadowThumb.setAttribute("style", "display:flex; ".concat(warpSryle)); // max-height:10000px 防止 font boosting

    shadowThumb.innerHTML = "<div style='flex: 1; overflow:hidden; max-height:10000px'>\n            <div style='overflow:hidden;text-overflow:ellipsis;white-space:nowrap;'>"
      .concat(
        emailInfo.from,
        "</div>\n            <div style='overflow:hidden;text-overflow:ellipsis;white-space:nowrap;'>"
      )
      .concat(
        emailInfo.date,
        "</div>\n        </div>\n        <div style='color:#FF7D2E; text-align:center; width: "
      )
      .concat(parseInt(fontSize) * 2 + 48, "px;' class='rm-show'>")
      .concat(moreText, "</div>");
    shadowFull.setAttribute("style", "display:block; ".concat(warpSryle));
    shadowFull.style.display = "none"; // float 做字体环绕效果

    shadowFull.innerHTML = "\n        <div style='color:#FF7D2E; float: right; text-align:center; width: "
      .concat(parseInt(fontSize) * 2 + 20, "px' class='rm-hide'>")
      .concat(
        lessText,
        "</div>\n        <div style='flex: 1; max-height:10000px'>\n            "
      )
      .concat(headerHtml, "\n        </div>\n        ");
    header.appendChild(shadowThumb);
    header.appendChild(shadowFull); // click

    header.addEventListener("click", function(e) {
      var target = e.target;
      var nodes = this.childNodes;

      if (/^rm-show/.test(target.className)) {
        nodes[0].style.display = "none";
        nodes[1].style.display = "block";
      } else if (/^rm-hide/.test(target.className)) {
        nodes[0].style.display = "flex";
        nodes[1].style.display = "none";
      }

      e.stopPropagation();
    });
  });
}

window.resetStyle = function() {
  var headerList = findMailHead(); // console.log(headerList);

  createShadow(headerList); // 参考旧代码, 有可能不隐藏

  setTimeout(function() {
    var hides = document.querySelectorAll(".rm-hide");

    for (var i = 0; i < hides.length; i++) {
      hides[i].click();
    }
  }, 1);
};
