(function(root, factory) {
  if (typeof define === "function" && define.amd) {
    define(function() {
      return factory(root);
    });
  } else if (typeof exports !== "undefined") {
    var Translate = factory(root);
    if (typeof module !== "undefined" && module.exports) {
      module.exports = Translate;
    }
    exports = Translate;
  } else {
    window.Translate = factory(root);
  }
})(this, function(win) {
  "use strict";
  var ua = navigator.userAgent;
  var is_ios = /(IPHONE|IPAD)/gi.test(ua);
  var is_android = /android/gi.test(ua);
  // 判断 是否是英文
  var isEn = ua.indexOf("Mail_En") > -1;
  var timer = null; // 定时器
  var options = {}; // 配置项
  var mailHeads = {};
  var translateText = []; // 翻译文本内容
  var paginationText = []; // 分页后的文本
  var isPause = false; // 是否暂停翻译
  var tsStatus = false; // 状态
  var Translate = {};
  var defaults = {
    // 是否分页调用翻译接口
    isPagination: false,
    // id前缀
    idPrefix: "_ts_id_",
    tagTsPrefix: "_ts",
    // 邮件折叠部分class
    foldClass: "_mail_fold_",
    tagHead: "_mail_head_",
    // 翻译源语言
    source: "zh",
    // 翻译目标语言
    target: "en",
    // 邮件头包含的文字内容
    headInclude: {
      zh: ["发件人", "收件人", "主题"],
      en: ["From", "To", "Subject"]
    },
    headIncludeAll: {
      zh: {
        发件人: "from",
        收件人: "to",
        抄送人: "cc2",
        抄送: "cc",
        日期: "date",
        发送时间: "date2",
        主题: "subject"
      },
      en: {
        From: "from",
        To: "to",
        Cc: "cc",
        Date: "date",
        Sent: "date2",
        Subject: "subject"
      }
    },
    // 不翻译的Tag
    exTagNames: [
      "NOSCRIPT",
      "SCRIPT",
      "STYLE",
      "CANVAS",
      "SVG",
      "BR",
      "IMG",
      "VIDEO",
      "AUDIO",
      "OBJECT",
      "CODE"
    ],
    // 中文正则
    zhReg: new RegExp(/[\u4e00-\u9fa5]/),
    // 文字正则 排除（特殊字符|数字|空格）
    textReg: new RegExp(
      /[^~!@#$%^&*()_\-+=<>?:{}|,.\/;\\[\]·~！@#￥%……&*（）——\-+={}|《》？：“”【】、；‘’，。、0-9\s]/
    ),
    reg: {
      en: new RegExp(/[a-zA-Z]/),
      zh: new RegExp(/[\u4e00-\u9fa5]/),
      jp: new RegExp(/[\u0800-\u4e00]/),
      kr: new RegExp(/[\uAC00-\uD7A3]/)
    },
    paginationMode: 1, // 0 数组长度分页 1 文字长度分页
    // 调用一次翻译最大数组长度
    maxPageSize: 10,
    // 文字最大数
    maxWordsSize: 2000,
    locale: {
      zh: {
        from: "发件人",
        to: "收件人",
        cc: "抄送",
        cc2: "抄送人",
        date: "日期",
        date2: "发送时间",
        subject: "主题"
      },
      en: {
        from: "From",
        to: "To",
        cc: "Cc",
        cc2: "Cc",
        date: "Date",
        date2: "Sent",
        subject: "Subject"
      }
    },
    // 邮件折叠翻译文字
    foldLocale: {
      zh: {
        show: "详情",
        hide: "隐藏"
      },
      en: {
        show: "Details",
        hide: "Hide"
      }
    }
  };
  var extend = function(target) {
    if (!arguments.length) {
      return arguments[0] || {};
    }
    for (var p in arguments[1]) {
      target[p] = arguments[1][p];
    }
    if (arguments.length > 2) {
      var moreArgs = [target].concat(Array.prototype.slice.call(arguments, 2));
      return extend.apply(null, moreArgs);
    } else {
      return target;
    }
  };

  /********** dom处理 **********/
  /********** 递归查找邮件头并标记 **********/
  // 递归查找邮件头并标记
  var findMailHead = function(nodes) {
    var len = nodes.length;
    for (var i = 0; i < len; i++) {
      var node = nodes[i];
      // 排除不翻译的nodeType
      if (
        node.nodeType === 3 ||
        options.exTagNames.indexOf(node.tagName) > -1
      ) {
        continue;
      }
      var isFrom = isIncludeFrom(node);
      var isHeadText = false;
      if (isFrom) isHeadText = isIncludeHeadText(node);
      if (isFrom && isHeadText) {
        // 找出邮件头容器
        var headNode = findMailHeadContainer(node);
        if (!headNode.getAttribute(options.tagHead)) {
          var id = options.tagHead + isFrom;
          // 设置标记
          headNode.setAttribute("class", headNode.className + " " + id);
          headNode.setAttribute(options.tagHead, id);
          mailHeads.push(headNode);
        }
        // 处理 邮件头包含在正文情况
        if (node.childNodes.length > 1) {
          var otherNodes = [];
          for (var m = 0; m < node.childNodes.length; m++) {
            var other = node.childNodes[m];
            if (
              m > 0 &&
              other.nodeType === 1 &&
              mailHeads.indexOf(other) === -1
            ) {
              otherNodes.push(other);
            }
          }
          findMailHead(otherNodes);
        }
      } else {
        findMailHead(node.childNodes);
      }
    }
  };
  // 是否是包含邮件头内容
  var isIncludeHeadText = function(node) {
    var isInclude = false;
    var innerText = nodeText(node);
    if (!innerText) return false;
    // 是否包含b标签
    if (!/<b/.test(node.innerHTML)) return false;
    // 中文头
    var textZh = options.headInclude.zh;
    for (var i = 0; i < textZh.length; i++) {
      if (innerText.indexOf(textZh[i]) === -1) {
        isInclude = false;
        break;
      } else {
        isInclude = "zh";
      }
    }
    if (!isInclude) {
      // 英文头
      var textEn = options.headInclude.en;
      for (var i = 0; i < textEn.length; i++) {
        if (innerText.indexOf(textEn[i]) === -1) {
          isInclude = false;
          break;
        } else {
          isInclude = "en";
        }
      }
    }
    return isInclude;
  };
  // 找到邮件头容器
  var findMailHeadContainer = function(node) {
    var container = node;
    var nodes = node.childNodes;
    var len = nodes.length;
    for (var i = 0; i < len; i++) {
      var child = nodes[i];
      if (isIncludeHeadText(child)) {
        container = findMailHeadContainer(child);
        break;
      }
    }
    return container;
  };
  // 遍历dom第一个最深的节点，查找是否包含 发件人
  var isIncludeFrom = function(node) {
    var rlt = false;
    if (node.nodeType === 3 && node.nodeValue) {
      // 中文发件人
      if (node.nodeValue.indexOf(options.headInclude.zh[0]) > -1) {
        rlt = "zh";
      } else if (node.nodeValue.indexOf(options.headInclude.en[0]) > -1) {
        //英文发件人
        rlt = "en";
      }
    } else if (node.childNodes.length > 0) {
      var nodes = node.childNodes;
      var len = nodes.length;
      for (var i = 0; i < len; i++) {
        // 排除隐含br的节点
        var text =
          nodes[i].nodeType === 3 ? nodes[i].nodeValue : nodes[i].innerHTML;
        if (/\S+/.test(text)) {
          rlt = isIncludeFrom(nodes[i]);
          break;
        }
      }
    }
    return rlt;
  };
  // 找出邮件头翻译内容
  var findMailHeadTranslateNode = function(nodes, pid, lang) {
    pid = pid || 0;
    var len = nodes.length;
    for (var i = 0; i < len; i++) {
      var node = nodes[i];
      if (!node) continue;
      if (
        node.nodeType === 3 &&
        /\S+/.test(node.nodeValue) &&
        parentNodeExitB(node)
      ) {
        var text = node.nodeValue;
        var headIncludeAll = options.headIncludeAll[lang];
        for (var k in headIncludeAll) {
          if (text.indexOf(k) > -1) {
            // 找出主题内容
            if (k === options.headInclude[lang][2] && node.parentNode) {
              var os = parentBNextSibling(node.parentNode);
              var os2 = parentSubjectNextSibling(os.pn);
              var obArr = [os, os2];
              var subject = "";
              var tsText = "";
              var sid = "";
              for (var m = 0; m < 2; m++) {
                var ob = obArr[m];
                if (ob.sb) {
                  sid = "subject_" + pid;
                  subject += findMailHeadSubject(ob.sb);
                  // sid节点已存在
                  var sidDom = document.getElementsByClassName(sid);
                  if (sidDom.length > 0) {
                    sidDom[0].innerHTML = subject;
                    break;
                  }
                  tsText = subject.replace(/&nbsp;/g, " ");
                  var sp = window.document.createElement("span");
                  sp.className = sid;
                  // 开始为冒号 + 主题后面没有冒号 冒号加粗
                  if (/^:/.test(tsText) && text.indexOf(":") === -1) {
                    tsText = tsText.replace(/^:/, "");
                    var sb = window.document.createElement("b");
                    sb.innerHTML = ":";
                    sp.innerHTML = tsText;
                    // 插入b
                    insertAfter(sb, ob.pn);
                    // 将拼接的主题内容插入主题标签之后
                    insertAfter(sp, sb);
                  } else {
                    sp.innerHTML = tsText;
                    // 将拼接的主题内容插入主题标签之后
                    insertAfter(sp, ob.pn);
                  }
                }
              }
              if (options.textReg.test(subject)) {
                // 记录数据
                translateText.push({
                  tsId: sid,
                  text: tsText,
                  textNodes: [{ id: sid, text: tsText }]
                });
              }
            }
            var np = window.document.createElement("span");
            np.innerHTML = k;
            np.className = options.tagHead + headIncludeAll[k];
            var nhtm = node.nodeValue.replace(k, np.outerHTML);
            node.parentNode.innerHTML = nhtm;
            break;
          }
        }
      } else {
        findMailHeadTranslateNode(node.childNodes, pid + "_" + i, lang);
      }
    }
  };
  // 容器是否是B标签
  var parentNodeExitB = function(node) {
    var rlt = false;
    if (!node.parentNode) return rlt;
    if (node.parentNode.tagName === "B") {
      rlt = true;
    } else if (node.parentNode.tagName === "SPAN") {
      rlt = parentNodeExitB(node.parentNode);
    }
    return rlt;
  };
  // 找出节点的父级B标签 的兄弟节点
  var parentBNextSibling = function(node) {
    var rlt = {
      sb: null, //b兄弟节点
      pn: null //b
    };
    if (!node) return rlt;
    if (node.tagName === "B") {
      rlt.sb = node.nextSibling;
      rlt.pn = node;
    } else {
      rlt = parentBNextSibling(node.parentNode);
    }
    return rlt;
  };
  // 找出主题节点的父级标签 的兄弟节点,取出可能存在的其它主题 截止邮件头部容器
  var parentSubjectNextSibling = function(node) {
    var rlt = {
      sb: null, //b兄弟节点
      pn: null //b
    };
    if (!node) return rlt;
    var pnode = node.parentNode;
    if (
      pnode &&
      pnode.className &&
      pnode.className.indexOf(options.tagHead) > -1
    ) {
      if (node.tagName === "B") {
        return rlt;
      }
      rlt.sb = node.nextSibling;
      rlt.pn = node;
    } else {
      rlt = parentSubjectNextSibling(node.parentNode);
    }
    return rlt;
  };
  // 找出邮件头主题
  var findMailHeadSubject = function(nextSibling) {
    var text = "";
    if (!nextSibling) return text;
    if (nextSibling.nodeType === 3) {
      text += nextSibling.nodeValue;
    } else {
      text += nextSibling.innerText;
    }
    if (nextSibling.nextSibling) {
      text += findMailHeadSubject(nextSibling.nextSibling);
    }
    // 删除自身
    nextSibling.parentNode && nextSibling.parentNode.removeChild(nextSibling);
    return text;
  };
  /********** 递归查找邮件头并标记 **********/
  /********** 标记文本节点 **********/
  // 找出非空的文本节点 并标记
  var findTextNodes = function(nodes, pid) {
    var len = nodes.length;
    for (var i = 0; i < len; i++) {
      var cid = pid + "_" + i;
      var node = nodes[i];
      if (options.exTagNames.indexOf(node.tagName) > -1) {
        continue;
      }
      if (node.nodeType === 3) {
        // 非空节点 进行标记
        if (/\S+/.test(node.nodeValue)) {
          var textNode = textToSpan(node);
          var oclass =
            textNode.className || textNode.parentNode.className || "";
          if (oclass) oclass += " ";
          if (textNode.nodeType === 3) {
            textNode.parentNode.setAttribute("class", oclass + cid);
            textNode.parentNode.setAttribute(options.tagTsPrefix, cid);
          } else {
            textNode.setAttribute("class", oclass + cid);
            textNode.setAttribute(options.tagTsPrefix, cid);
          }
        }
      }
      if (node.className && node.className.indexOf(options.tagHead) > -1) {
        // 排除邮件节点
        continue;
      }
      // 排除折叠节点
      if (node.nodeType === 1 && node.getAttribute("fid")) continue;
      if (node.childNodes.length > 0) {
        findTextNodes(node.childNodes, cid);
      }
    }
  };
  // 查找文本，按规则合并
  var findTranslateText = function(nodes) {
    var len = nodes.length;
    for (var i = 0; i < len; i++) {
      var node = nodes[i];
      if (options.exTagNames.indexOf(node.tagName) > -1) {
        continue;
      }
      // 排除邮件头节点
      if (node.className && node.className.indexOf(options.tagHead) > -1) {
        continue;
      }
      // 排除折叠节点
      if (node.nodeType === 1 && node.getAttribute("fid")) continue;
      // 独立文本节点
      if (
        (!node.children && node.nodeType === 1) ||
        (node.children && node.children.length === 0)
      ) {
        var tagId = node.getAttribute(options.tagTsPrefix);
        var nodeText = node.innerText;
        if (tagId && options.textReg.test(nodeText)) {
          translateText.push({
            tsId: tagId,
            text: nodeText.replace(/\s+/, " "),
            textNodes: [{ id: tagId, text: nodeText }]
          });
        }
      } else if (node.childNodes.length > 0) {
        // 继续往下遍历
        getWordsByNodes(node.childNodes);
      }
    }
  };
  // 一句话
  var getWordsByNodes = function(nodes) {
    var len = nodes.length;
    for (var i = 0; i < len; i++) {
      var node = nodes[i];
      if (options.exTagNames.indexOf(node.tagName) > -1) {
        continue;
      }
      // 排除邮件头节点
      if (node.className && node.className.indexOf(options.tagHead) > -1) {
        continue;
      }
      // 排除折叠节点
      if (node.nodeType === 1 && node.getAttribute("fid")) continue;
      var innerHtml = node.innerHTML;
      if (!innerHtml) continue;
      var tagId = node.getAttribute(options.tagTsPrefix);
      var nodeText = node.innerText;
      // 没有子tag 只有text节点
      if (tagId && node.children && node.children.length === 0) {
        // 是否包含中文
        if (nodeText && options.textReg.test(nodeText)) {
          translateText.push({
            tsId: tagId,
            text: nodeText,
            textNodes: [{ id: tagId, text: nodeText }]
          });
        }
      }
      // 包含a,p，h 以a,p，h断句 优先级 h > p > a
      else if (
        node.tagName === "P" ||
        node.tagName === "A" ||
        /^H/.test(node.tagName)
      ) {
        var words = pTagWords(node);
        // 是否包含中文
        if (words.text && options.textReg.test(words.text)) {
          words.text = words.text.replace(/\s+/, " ");
          for (var m = 0; m < words.textNodes.length; m++) {
            // 以第一个汉字节点作为翻译插入节点id
            if (options.textReg.test(words.textNodes[m].text)) {
              words.tsId = words.textNodes[m].id;
              break;
            }
          }
          translateText.push(words);
        }
      } else {
        getWordsByNodes(node.childNodes);
      }
    }
  };
  // 传入p标签过滤出 一句话
  var pTagWords = function(node) {
    var maps = {
      text: "", //p节点所有内容
      textNodes: [] //有内容的节点id + 内容
    };
    var ps = node.childNodes;
    var len = ps.length;
    for (var i = 0; i < len; i++) {
      var p = ps[i];
      // 排除不翻译的nodeType
      if (options.exTagNames.indexOf(p.tagName) > -1) {
        continue;
      }
      // 排除邮件头节点
      if (p.className && p.className.indexOf(options.tagHead) > -1) {
        continue;
      }
      // br 标签会识别为换行text
      if (p.nodeType === 3 || p.nodeType === 8) continue;
      var tagId = p.getAttribute(options.tagTsPrefix);
      // 没有其它tag 只有text
      if (tagId && p.children && p.children.length === 0) {
        maps.text += p.innerText;
        maps.textNodes.push({
          id: tagId,
          text: p.innerText
        });
      } else if (p.childNodes.length > 0) {
        var child = pTagWords(p);
        maps.text += child.text;
        maps.textNodes = maps.textNodes.concat(child.textNodes);
      }
    }
    return maps;
  };
  // text非独立节点加上span
  var textToSpan = function(node) {
    // 非独立text节点 含有兄弟节点
    if (
      node.previousSibling ||
      node.nextSibling ||
      node.parentNode.nodeName === "BODY"
    ) {
      // 针对该文本节点创建一个新的span存放
      var np = window.document.createElement("span");
      // np.id = pid;
      np.innerHTML = node.nodeValue;
      // 替换原有text节点
      node.parentNode.replaceChild(np, node);
      return np;
    }
    return node;
  };
  // 获取节点text内容
  var nodeText = function(node) {
    if (node.nodeType === 3) return node.nodeValue;
    return node.innerText;
  };
  // 插入兄弟节点
  var insertAfter = function(newElement, targetElement) {
    var parent = targetElement.parentNode;
    if (parent.lastChild == targetElement) {
      // 如果最后的节点是目标元素，则直接添加。因为默认是最后
      parent.appendChild(newElement);
    } else {
      parent.insertBefore(newElement, targetElement.nextSibling); //如果不是，则插入在目标元素的下一个兄弟节点 的前面。也就是目标元素的后面
    }
  };
  /********** 标记文本节点 **********/
  // 翻译邮件头部
  var toTsHead = function() {
    var target = isEn ? "en" : "zh";
    //翻译折叠文字内容
    if (typeof window.resetStyle === "function") {
      var lang2 = options.foldLocale[target];
      for (var m in lang2) {
        var doms2 = document.getElementsByClassName("rm-" + m);
        for (var i = 0; i < doms2.length; i++) {
          var dom = doms2[i];
          dom.innerHTML = lang2[m];
        }
      }
    }
    // 翻译邮件头部
    var lang = options.locale[target];
    for (var k in lang) {
      var doms = document.getElementsByClassName(options.tagHead + k);
      for (var i = 0; i < doms.length; i++) {
        var dom = doms[i];
        dom.innerHTML = lang[k];
      }
    }
  };
  // 将翻译更新到dom
  var toTsDom = function(index) {
    // 暂停渲染
    if (isPause) {
      console.log("渲染已暂停");
      return;
    }
    var arr = paginationText[index];
    var len = arr.length;
    for (var i = 0; i < len; i++) {
      if (!arr[i][options.target]) continue;
      // 循环该p Tag 所有文字节点,清空内容
      var textNodes = arr[i].textNodes;
      for (var j = 0; j < textNodes.length; j++) {
        var d = document.getElementsByClassName(textNodes[j].id)[0];
        if (d) d.innerHTML = "";
      }
      // 将翻译内容插入到翻译节点
      var dd = document.getElementsByClassName(arr[i].tsId)[0];
      if (dd) {
        dd.innerHTML = arr[i][options.target];
      } else {
        console.log(arr[i]);
      }
    }
    window["ts_count_" + options.target] += 1;
    if (window["ts_count_" + options.target] >= paginationText.length) {
      window["ts_count_" + options.target] = 0;
      nativeToast("翻译完成");
    }
  };
  // 还原dom结构
  var toOldDom = function() {
    var len = paginationText.length;
    for (var i = 0; i < len; i++) {
      var arr = paginationText[i];
      var alen = arr.length;
      // 将所有p还原
      for (var j = 0; j < alen; j++) {
        var p = arr[j];
        // 还原所有节点
        for (var k = 0; k < p.textNodes.length; k++) {
          var d = document.getElementsByClassName(p.textNodes[k].id)[0];
          if (d) d.innerHTML = p.textNodes[k].text;
        }
      }
    }
  };
  /********** dom处理 **********/

  /********** 数组操作 **********/
  // 分页的数组 合并为新数组 [[],[],...]
  var paginationArray = function(array) {
    var rlt = [];
    var pageArr = [];
    var page = 1;
    var size = options.maxPageSize;
    var len = array.length;
    for (var i = 0; i < len; i++) {
      if (i < page * size) {
        pageArr.push(array[i]);
      } else {
        rlt.push(pageArr);
        page++;
        pageArr = [];
        pageArr.push(array[i]);
      }
    }
    rlt.push(pageArr);
    return rlt;
  };
  // 文字长度分页
  var paginationWords = function(array) {
    var rlt = [];
    var pageArr = [];
    var len = array.length;
    var text = "";
    for (var i = 0; i < len; i++) {
      text += array[i].text;
      if (text.length > options.maxWordsSize) {
        if (pageArr.length) rlt.push(pageArr);
        text = "";
        pageArr = [];
        pageArr.push(array[i]);
      } else {
        pageArr.push(array[i]);
      }
    }
    rlt.push(pageArr);
    return rlt;
  };
  // 翻译数据更新到缓存
  var toPaginationText = function(arr, index, target) {
    var len = arr.length;
    for (var i = 0; i < len; i++) {
      paginationText[index][i][target] = arr[i];
    }
    // 更新到dom
    toTsDom(index);
  };
  /********** 数组操作 **********/
  // 翻译所有
  var nativeTranslateAll = function(start, source, target) {
    // 起始页码
    start = start || 0;
    function allToNative(index, tArr, s, t) {
      App.call("emailTranslationAll", JSON.stringify(tArr), s, t, function(
        res
      ) {
        var tsArr = [];
        if (is_android) res = decodeURIComponent(res.replace(/\+/g, "%20"));
        if (typeof res === "string") res = JSON.parse(res);
        if (parseInt(res.code) === 200) {
          tsArr = res.body.targetText || [];
        } else {
          console.log(res.message);
        }
        if (tsArr.length > 0) toPaginationText(tsArr, index, t);
      });
    }
    var len = paginationText.length;
    for (var j = start; j < len; j++) {
      var tsArray = paginationText[j];
      if (!tsArray) continue;
      // 已经含有翻译内容
      if (tsArray.length > 0 && tsArray[0][target]) {
        // 直接渲染
        toTsDom(j);
      } else {
        // 处理文本内容
        var rlt = [];
        for (var i = 0; i < tsArray.length; i++) {
          rlt.push(tsArray[i].text.replace(/\|/g, "/"));
        }
        // 调用接口
        allToNative(j, rlt, source, target);
      }
    }
  };
  // 迭代调用native翻译接口
  var nativeTranslate = function(index, source, target) {
    var tsArray = paginationText[index];
    if (!tsArray) return;
    if (!isPause && tsArray && tsArray.length > 0 && tsArray[0][target]) {
      toTsDom(index);
      if (options.isPagination) {
        // 分页调用接口翻译内容
        nativeTranslate(index + 1, source, target);
      } else {
        // 剩下的页面 全部翻译
        nativeTranslateAll(1, source, target);
      }
      return;
    }
    var rlt = [];
    for (var i = 0; i < tsArray.length; i++) {
      rlt.push(tsArray[i].text.replace(/\|/g, "/"));
    }
    App.call(
      "emailTranslationAll",
      JSON.stringify(rlt),
      source,
      target,
      function(res) {
        var tsArr = [];
        if (is_android) res = decodeURIComponent(res.replace(/\+/g, "%20"));
        if (typeof res === "string") res = JSON.parse(res);
        if (parseInt(res.code) === 200) {
          tsArr = res.body.targetText || [];
        } else {
          console.log(res.message);
        }
        if (!isPause && index < paginationText.length) {
          if (tsArr.length > 0) toPaginationText(tsArr, index, target);
          if (options.isPagination) {
            // 分页调用翻译接口
            if (index + 1 < paginationText.length)
              nativeTranslate(index + 1, source, target);
          } else if (paginationText.length > 1) {
            // 剩下的页面 全部翻译
            nativeTranslateAll(1, source, target);
          }
        }
      }
    );
  };
  // 调用native toast
  var nativeToast = function(msg) {
    App.call("emailTranslationToast", msg, 0, function() {});
  };
  /******* 模拟翻译 ******/
  // 测试 获取英文翻译内容
  var nativeTranslate2 = function(index, source, target) {
    var tsArray = paginationText[index];
    if (!isPause && tsArray && tsArray.length > 0 && tsArray[0][target]) {
      toTsDom(index);
      // 递归调用接口
      nativeTranslate2(index + 1, source, target);
      return;
    }
    setTimeout(function() {
      if (!isPause && index < paginationText.length) {
        var enArr = getTs(tsArray);
        toPaginationText(enArr, index, target);
        // 递归调用接口
        nativeTranslate2(index + 1, source, target);
      }
    }, 100);
  };
  var getTs = function(a) {
    var r = [];
    for (var i = 0; i < a.length; i++) {
      r.push("EN>" + a[i].text);
    }
    return r;
  };
  /******* 模拟翻译 ******/
  // 文档初始化
  Translate.init = function(opts) {
    if (tsStatus) return;
    tsStatus = false;
    mailHeads = [];
    translateText = [];
    paginationText = [];
    options = extend({}, defaults, opts);
    var doc = window.document.body.childNodes;
    // 查找邮件头
    findMailHead(doc);
    // 找出邮件头部可翻译内容
    var len = mailHeads.length;
    for (var i = 0; i < len; i++) {
      var hNode = mailHeads[i];
      var attr = hNode.getAttribute(options.tagHead) || "zh";
      var lang = attr.replace(options.tagHead, "");
      findMailHeadTranslateNode(hNode.childNodes, i, lang);
    }
    //邮件折叠部分可翻译内容
    var foldDoms = document.getElementsByClassName(options.foldClass);
    for (var lang in options.headInclude) {
      findMailHeadTranslateNode(foldDoms, 0, lang);
    }
    // 找到所有正文文本节点并标记
    findTextNodes(doc, options.tagTsPrefix);
    // 记录所有可翻译的文本对象
    findTranslateText(doc);
    // 分页
    if (options.paginationMode === 1) {
      // 文字长度分页
      paginationText = paginationWords(translateText);
    } else {
      //数组长度分页
      paginationText = paginationArray(translateText);
    }
    tsStatus = options.source;
  };
  // 翻译 target 目标语言
  Translate.toDo = function(source, target) {
    if (paginationText.length === 0) {
      console.log("没有可翻译的内容！");
      return;
    }
    window["ts_count_" + target] = 0;
    source = source || options.source;
    target = target || options.target;
    options.source = source;
    options.target = target;
    isPause = false;
    // 防抖
    clearTimeout(timer);
    timer = setTimeout(function() {
      // 防未初始化完成
      if (!tsStatus) {
        Translate.toDo();
      } else {
        console.log("开始翻译\r\n");
        // 翻译头部
        toTsHead(target);
        tsStatus = target;
        // 分页调用接口翻译内容
        nativeTranslate(0, source, target);
      }
    }, 100);
  };
  Translate.stop = function() {
    isPause = true;
    tsStatus = "stop";
  };
  Translate.reset = function() {
    // if (tsStatus === options.source) return;
    isPause = true;
    console.log("还原dom\r\n");
    // 还原头部
    toTsHead();
    // 还原内容
    toOldDom();
    tsStatus = options.source;
  };
  Translate.reInit = function() {
    tsStatus = false;
    mailHeads = [];
    translateText = [];
    paginationText = [];
    clearTimeout(timer);
  };
  return Translate;
});
