var RE = {};
RE.oldRange = null;
RE.oldCulY = 0;
RE.oldCulX = 0;
var PARichEditor = window.PARichEditor;

RE.editor = document.getElementById('editor');

PARichEditor.init({
    element: RE.editor,
    defaultParagraphSeparator: 'br',
    onHtmlChange: function (html) {
            if (window.writeCallback) {
                window.writeCallback.onTextChanged(PARichEditor.getHtml());
            }
        },
        onStateChange: function (actionName, result) {
            if (window.writeCallback) {
                window.writeCallback.onActionButtonStateChange(actionName, result);
                RE.checkNeedScrollY(false);
            }
        },
        onClickResult: function (actionName, result) {
            if (window.writeCallback) {
                window.writeCallback.onActionButtonClickResult(actionName, result);
            }
        }
});

RE.getYCoods = function () {
    var sel = window.getSelection(),
        range,
        isSpan = false,
        top = 0,
        bottom = 0,
        height = 0,
        left = 0;
    if (sel.rangeCount) {
        range = sel.getRangeAt(0).cloneRange();
        var rect;
        if (range.getClientRects().length > 0) {
            range.collapse(true);
            rect = range.getClientRects()[0];
            top = rect.top;
            height = rect.height;
            bottom = rect.bottom;
            left = rect.left;
        } else {
            var currentNode = range.endContainer,
                currentOffset = range.endOffset,
                children = Array.prototype.slice.call(currentNode.childNodes);
            if (children.length > 0 && currentOffset == 0) {
                currentOffset = 1;
            }

            if (children[currentOffset - 1]) {
                var childNode = children[currentOffset - 1];
                //text没有getBoundingClientRect()函数，寻找相邻的node计算，相邻的没有就需要插入一个span
                //计算，插入span又会触发状态改变会进入死循环检测滚动，延迟10毫秒设置checkNeedScrollYing为false解决
                if (childNode.nodeType == 3) {
                    if (childNode.nextSibling && childNode.nextSibling.nodeType != 3) {
                        rect = childNode.nextSibling.getBoundingClientRect();
                    } else if (childNode.previousSibling && childNode.previousSibling.nodeType != 3) {
                        rect = childNode.previousSibling.getBoundingClientRect();
                    } else {
                        isSpan = true;
                        var selectionEnd = document.createElement("span");
                        var endRange = document.createRange();
                        endRange.setStart(currentNode, currentOffset);
                        endRange.insertNode(selectionEnd);
                        rect = selectionEnd.getBoundingClientRect();
                        var spanParent = selectionEnd.parentNode;
                        spanParent.removeChild(selectionEnd);
                        spanParent.normalize();
                        sel.removeAllRanges();
                        sel.addRange(range);
                    }
                } else {
                    rect = childNode.getBoundingClientRect();
                }
            } else if ("id_editor_content" == currentNode.id) {
                rect = {};
                rect.top = 0;
                rect.height = 20;
                rect.bottom = 0;
                rect.left = 0;
            } else {
                rect = currentNode.getBoundingClientRect();
            }
            top = rect.top;
            height = rect.height;
            bottom = rect.bottom;
            left = rect.left;
        }
    }
    if (isSpan) {
        setTimeout(function () {
            RE.checkNeedScrollYing = false;
        }, 10);
    } else {
        RE.checkNeedScrollYing = false;
    }

    return {
        top: top,
        bottom: bottom,
        height: height,
        left: left,
    };
}
RE.checkNeedScrollYing = false;
RE.checkNeedScrollY = function (forceCheck) {
    if (RE.checkNeedScrollYing || !document.hasFocus()) {
        return;
    }
    RE.checkNeedScrollYing = true;
    try {
        var selection = window.getSelection();
        var range;
        if (selection.rangeCount > 0) {
            range = selection.getRangeAt(0);
        }
        var needCheck = range && range.startOffset == range.endOffset && (forceCheck || (range != RE.oldRange || range.endOffset != RE.oldRange.endOffset));
        var YCoods;
        var flag = false;
        if (!needCheck) { //先用Range判断是否需要检测，Range一样使用光标位置计算是否需要检测，优化代码
            YCoods = RE.getYCoods();
            flag = true;
            needCheck = RE.oldCulY != YCoods.bottom || RE.oldCulX != YCoods.left;
            RE.oldCulY = YCoods.bottom;
            RE.oldCulX = YCoods.left;
        }

        if (needCheck) {
            RE.oldRange = range;
            var scrollDataStr = window.writeCallback.getScrollData();
            if (scrollDataStr) {
                var scrollData = JSON.parse(scrollDataStr);
                if (!flag) {
                    YCoods = RE.getYCoods();
                }
                RE.oldCulY = YCoods.bottom;
                RE.oldCulX = YCoods.left;
                var scrollYBootom = YCoods.bottom * scrollData.density + scrollData.webTop + scrollData.offSet - scrollData.scrollHeight - scrollData.availHeight; //输入内容自动下滚
                var scrollYTop = (scrollData.scrollHeight - scrollData.webTop) - (YCoods.top * scrollData.density); //删除内容自动上滚
                console.log(range.startOffset + " " + (YCoods.bottom * scrollData.density) + " " + scrollDataStr + " " + scrollYBootom + " " + scrollYTop);

                if (scrollYBootom > 0) {
                    window.writeCallback.needScrollBy(scrollYBootom + (20 * scrollData.density)); //自动下滚
                } else if (scrollYTop > 0) {
                    window.writeCallback.needScrollBy(-(scrollYTop + (20 * scrollData.density))); //自动上滚
                }
            }
        }
    } catch (e) {
        RE.checkNeedScrollYing = false;
    }
}

RE.setHtml = function (thml) {
    PARichEditor.setHtml(thml);
    //给rtf格式的大附件下载链接添加title=PaicBigAttachmentDownload
    judgeBigAttachment();
    addpTagAttr();
}

RE.setNodeRangeEndById = function (id){
 var node = document.getElementById(id);
 PARichEditor.setFocusNodeRangeEnd(node);
}

RE.setPadding = function (left, top, right, bottom) {
    var editor = PARichEditor.getElement();
    editor.style.paddingLeft = left;
    editor.style.paddingTop = top;
    editor.style.paddingRight = right;
    editor.style.paddingBottom = bottom;
}

RE.setHeight = function (size) {
    PARichEditor.setHeight(size);
}

RE.setPlaceholder = function (placeholder) {
    PARichEditor.setPlaceholder(placeholder);
}

RE.doClickActionButton = function (actionBunName) {
    PARichEditor.doClickActionButton(actionBunName);
}

RE.undo = function () {
    document.execCommand('undo', false, null);
}

RE.redo = function () {
    document.execCommand('redo', false, null);
}

RE.selectAll = function () {
    PARichEditor.getElement().focus();
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.selectNodeContents(PARichEditor.getElement());
    selection.addRange(range);
}

RE.insertHTML = function (html) {
    PARichEditor.insertHTML(html);
}

RE.focus = function (pos) {
    PARichEditor.getElement().focus();
    var range = document.createRange();
    range.selectNodeContents(PARichEditor.getElement());
    range.collapse(true);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
}

RE.focusEnd = function (pos) {
    //RE.editor.focus();
    //RE.editor.setSelectionRange(pos,pos);
    var range = document.createRange();
    range.selectNodeContents(PARichEditor.getElement());
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    PARichEditor.getElement().focus();
}

RE.onScrollChanged = function (scrollHeight, availHeight, webTop, offSet) {
    var lazyLoadImgs = document.querySelectorAll('.lazyLoad');
    if (lazyLoadImgs) {
        for (var x = 0; x < lazyLoadImgs.length; x++) {
            var el = lazyLoadImgs[x];
            var contentTop = el.offsetTop;
            //img top 距离可视区域底部的高度
            if ((contentTop + webTop) - scrollHeight - availHeight < offSet) {
                RE.doLazyLoadImg(el);
            }
        }
    }
}

RE.doLazyLoadImg = function (el) {
    if (el.hasAttribute('replacesrckey')) {
        var key = el.getAttribute('replacesrckey');
        var obj_class = ' ' + el.className + ' ';
        obj_class = obj_class.replace(/(\s+)/gi, ' ');
        removed = obj_class.replace(' lazyLoad ', ' lazyLoading ');
        el.className = removed;
        window.writeCallback.doLazyLoadImg(key);
    }
}

RE.onLoadLazyImg = function (i, replacesrckey, src) {
    var el = document.getElementById("lazyLoad_" + i);
    // 屏蔽图片长按弹出复制菜单
    el.addEventListener("touchstart", function (e) {
     e.preventDefault();
    }, false);
    if (el.hasAttribute('replacesrckey')) {
        var key = el.getAttribute('replacesrckey');
        if (key == replacesrckey) {
            el.src = src;
        }
    }
}

//给大附件标签添加丢失的css属性
function addpTagAttr() {
    var tag = 'p';
    var cls = 'class';
    var clsValue = 'MsoNormal';
    var attr = 'style';
    var attrValue = 'margin: 0cm 0cm 0.0001pt; line-height: 21px; font-size: medium; font-family: 等线;';
    var items = document.getElementsByTagName(tag);
    for (item of items) {
        if (item.getAttribute(cls) == clsValue) {
            if (item.getAttribute(attr) == null) {
                item.setAttribute(attr, attrValue);
            } else if (item.getAttribute(attr).indexOf(attrValue) == -1) {
                item.setAttribute(attr, attrValue + item.getAttribute(attr));
            }
        }
    }
};

//给rtf下的大附件下载链接添加title=PaicBigAttachmentDownload
function judgeBigAttachment() {
    var aTags = document.getElementsByTagName('a');
    for (item of aTags) {
        hrefVal = item.getAttribute('href');
        if (hrefVal.indexOf('dl=') != -1 && hrefVal.indexOf('source=pc_outlook_bigattachment') != -1) {
            item.setAttribute('title', 'PaicBigAttachmentDownload');
        }
    }
}