(function(root, factory) {
  if (typeof define === "function" && define.amd) {
    define(function() {
      return factory(root);
    });
  } else if (typeof exports !== "undefined") {
    var Zoom = factory(root);
    if (typeof module !== "undefined" && module.exports) {
      module.exports = Zoom;
    }
    exports = Zoom;
  } else {
    window.Zoom = factory(root);
  }
})(this, function() {
  var ua = navigator.userAgent;
  var host = window.location.host;
  var type = window.publicImageTextType || 1; //1:普通图文 2:只带评论的图文 3:话题PK图文 4:投票图文
  var ignoreClass = "public-big-font-ignore"; //大字体忽略class
  var isAndroid = ua.toLocaleUpperCase().indexOf("ANDROID") !== -1;
  var PixelRatio = isAndroid ? 1 : window.devicePixelRatio;
  var reg = new RegExp(/FontSize_(\w+)/gi);
  var ma = ua.match(reg);
  var fontKey = "FontSize_stander";
  if (ma) {
    fontKey = ma[0] || "FontSize_stander";
    console.log(fontKey);
  }
  // 大字体放大系数 +
  var fontsKeys = {
    FontSize_stander: 0,
    FontSize_medium: 2 * PixelRatio,
    FontSize_large: 4 * PixelRatio,
    FontSize_extraLarge: 6 * PixelRatio
  };
  // 字体增加系数
  var fontAddNum = fontsKeys[fontKey];
  function Zoom() {}
  Zoom.prototype.init = function() {
    // 3:话题PK图文 4:投票图文 不使用大字体
    if (type > 2) return;
    host = window.location.host;
    // iobs地址 + 设置了大字体 使用此功能
    if (!/^(iobs|stg.iobs)/i.test(host) || fontAddNum === 0) {
      return;
    }
    this.setMetaViewport();
    var nodes = window.document.body.childNodes;
    this.findTextNodes(nodes, 0);
  };
  Zoom.prototype.findTextNodes = function(nodes, pid) {
    var len = nodes.length;
    for (var i = 0; i < len; i++) {
      var node = nodes[i];
      // 跳过ignore节点
      if (node.className && node.className.indexOf(ignoreClass) > -1) {
        continue;
      }
      if (node.nodeType !== 3 && node.childNodes.length > 0) {
        if (node) this.findTextNodes(node.childNodes, pid + "_" + i);
        continue;
      }
      if (node.nodeType === 3) {
        // 非空节点 进行标记
        if (/\S+/.test(node.nodeValue)) {
          var textNode = this.textToSpan(node);
          if (textNode.nodeType === 3) {
            this.setStyle(textNode.parentNode);
          } else {
            this.setStyle(textNode);
          }
        }
      }
    }
  };
  Zoom.prototype.textToSpan = function(node) {
    if (!node.previousSibling && !node.nextSibling) {
      return node;
    }
    // 针对该文本节点创建一个新的span存放
    var np = window.document.createElement("span");
    np.innerHTML = node.nodeValue;
    // 替换原有text节点
    node.parentNode.replaceChild(np, node);
    return np;
  };
  Zoom.prototype.getComputedStyle = function(node, styleStr) {
    styleStr = styleStr || "font-size";
    return window.getComputedStyle(node, null)[styleStr];
  };
  // 设置节点样式
  Zoom.prototype.setStyle = function(node) {
    // 节点原样式
    var oldStyle = node.getAttribute ? node.getAttribute("style") || "" : "";
    if (oldStyle) oldStyle += ";";
    var ofontSize = node.style["font-size"] || this.getComputedStyle(node);
    var nfontSize = parseInt(parseInt(ofontSize) + fontAddNum) + "px";
    var nLinHeight = this.getLineHeight(node, ofontSize);
    var height = this.getHeight(node);
    var nStyle =
      oldStyle + "font-size:" + nfontSize + ";line-height: " + nLinHeight + ";";
    // 原高度 小于 行高
    if (height < parseInt(nLinHeight)) {
      nStyle += "min-height: " + nLinHeight;
    }
    node.setAttribute("style", nStyle);
  };
  Zoom.prototype.getLineHeight = function(node, fontsize) {
    var lh = this.getComputedStyle(node, "line-height");
    if (lh.indexOf("px") > -1) {
      return parseInt(lh) + fontAddNum + "px";
    } else {
      var nLh = parseInt(lh) || fontsize;
      return nLh + fontAddNum + "px";
    }
  };
  Zoom.prototype.getHeight = function(node) {
    return parseInt(this.getComputedStyle(node, "height")) || 0;
  };
  // 移动端兼容标签设置
  Zoom.prototype.setMetaViewport = function() {
    var metas = document.getElementsByTagName("meta");
    var isviewport = false;
    for (var i = 0; i < metas.length; i++) {
      var meta = metas[i];
      if (meta.name === "viewport") {
        isviewport = true;
        break;
      }
    }
    if (!isviewport) {
      var mt = window.document.createElement("meta");
      mt.name = "viewport";
      mt.content =
        "width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=0";
      document.head.appendChild(mt);
    }
  };
  var _zoom = new Zoom();
  _zoom.init();
  return _zoom;
});
